import stableStringify from 'json-stable-stringify';
import { resolvePath } from '@knapsack/file-utils';
import { isFirstLetterCapital, memoize } from '@knapsack/utils';
import { isComponent } from './ng-type-guards.mjs';
async function importPkgUncached({ path, pkgPathAliases, resolveFrom = process.cwd(), }) {
    try {
        const mod = await import(path);
        return mod;
    }
    catch (e) {
        const { absolutePath } = await resolvePath({
            path,
            resolveFromDir: resolveFrom,
            pkgPathAliases,
        });
        return import(absolutePath);
    }
}
const importPkg = memoize(importPkgUncached, {
    // With promise option we indicate that we memoize a function that returns promise. The difference from natural behavior is that in case when promise was rejected with exception, the result is immediately removed from memoize cache, and not kept as further reusable result.
    promise: true,
    max: 10,
    maxAge: 15000,
    normalizer: (args) => stableStringify(args) || JSON.stringify(args),
});
export const clearImporterCache = () => importPkg.clear();
export async function importComponent({ path, alias = 'default', resolveFrom = process.cwd(), pkgPathAliases, }) {
    const pkg = await importPkg({
        path,
        resolveFrom,
        pkgPathAliases,
    });
    const it = pkg[alias];
    if (!it) {
        throw new Error(`Component ${alias} not found in ${path}`);
    }
    // @todo remove cast, do run time check
    return it;
}
export async function getComponentNamesFromPkg({ path, pkgPathAliases, resolveFrom, }) {
    const thePkg = await importPkg({
        path,
        resolveFrom,
        pkgPathAliases,
    });
    return (Object.entries(thePkg || {})
        // only allow names that start with a capital letter (i.e. Component names)
        // will exclude helper functions and custom hooks
        .filter(([exportName, exportValue]) => {
        if (exportName === 'default' || !isFirstLetterCapital(exportName)) {
            return false;
        }
        return isComponent(exportValue);
    })
        .map(([name]) => name));
}
//# sourceMappingURL=importer.mjs.map